import 'dart:async';
import 'log_level.dart';
import 'package:stack_trace/stack_trace.dart';

class LogRecord {
  final LogLevel level;
  final Object message;
  final String module;
  final DateTime time;
  final Object exception;
  final StackTrace stackTrace;
  final Zone zone;

  LogRecord(this.module, this.level, this.message,
      [this.exception, this.zone, this.stackTrace])
      : time = new DateTime.now();

  String format() {
    Chain chain = Chain.parse(stackTrace.toString());
    List<Frame> frames = chain.traces[0].frames;
    Frame callFrame = frames[2];
    String callStr = "${callFrame.member}(${callFrame.uri}:${callFrame.line}:${callFrame.column})";
    String output;
    if (exception != null){
      String exceptionStr = "\nthrow exception: $exception";
      output = "$time [$callStr][$module][${_getLogLevelName()}] - $message $exceptionStr";
    }else{
      output = "$time [$callStr][$module][${_getLogLevelName()}] - $message";
    }
    return output;
  }

  String _getLogLevelName() {
    switch (level) {
      case LogLevel.trace:
        return "T";
      case LogLevel.debug:
        return "D";
      case LogLevel.info:
        return "I";
      case LogLevel.warning:
        return "W";
      case LogLevel.error:
        return "E";
      case LogLevel.fatal:
        return "F";
      case LogLevel.crash:
        return "C";
    }
  }

  @override
  String toString() => '[${_getLogLevelName()}] $module: $message';
}
